<?php
require_once('object_class.php');

/**
 * A contact definition is used to identify someone who should
 * be contacted in the event of a problem on your network. This is
 * not always a user, which means not all contacts can log into the
 * Monitor GUI.
 */
class contact_class extends object_class
{
	/** object type */
	var $type = 'contact';

	/** Nagios' default settings */
	var $nagios_defaults = array(
		'host_notification_options' => array('n' => ''),
		'service_notification_options' => array('n' => ''),
		'host_notifications_enabled' => 1,
		'service_notifications_enabled' => 1,
		'can_submit_commands' => 0,
		'retain_status_information' => 1,
		'retain_nonstatus_information' => 1
	);

	/** If true, merge in user (access right) properties */
	protected $should_contain_user_class = false;

	/**
	 * @param $id = false
	 * @param $should_contain_user_class = false
	 */
	public function __construct($id = false, $should_contain_user_class = false) {
		$this->should_contain_user_class = $should_contain_user_class;
		parent::__construct($id);
	}

	public function get_properties()
	{
		parent::get_properties();
		if ($this->should_contain_user_class)
			$this->_assemble_user_class_properties($this->id);
		return $this->properties;
	}

	/**
	 * Set up this contact class' properties by merging them
	 * with those found in user_class->properties.
	 *
	 * @param $id
	 */
	private function _assemble_user_class_properties($id = false) {
		$user_class = new user_class($id);

		// Add the user_class' form fields to this instance of contact_class
		foreach($user_class->get_properties() as $property => $value) {
			if ($property === 'topic')
				continue;
			if(isset($value['username'])) {
				unset($value['username']);
			}
			$this->properties[$property] = array_merge($this->properties[$property], $value);
		}
		$this->properties['var_index']['enable_access'] = 'enable_access';
		$this->properties['default']['enable_access'] = false;

		if($id && 'new' != $id && isset($this->obj['contact_name'])) {
			// joins already saved user data when editing the contact
			$user = new user_class($this->obj['contact_name']);
			if($user->obj) {
				// use the properties of the possible existing and in
				// that case equally named user
				$this->obj = array_merge($this->obj, $user->obj);
			}
		}
	}

	/**
	 * Validate the object as sane prior to writing it to database.
	 * Either email or pager must be set for contacts, or nagios
	 * will complain when loading the config.
	 * @return boolean True on success, false on errors.
	 */
	function validate_object()
	{
		parent::validate_object();
		return empty($this->validation_error_list);
	}

	function save_object($user = true) {
		// remove the merged properties used for form generation
		$userpasswd = new user_class();
		foreach($userpasswd->properties as $property => $value) {
			if ($property === 'topic')
				continue;
			foreach($value as $sub_property => $real_value) {
				unset($this->properties[$property][$sub_property]);
			}
		}
		if($this->should_contain_user_class) {
			// ... and add the values the user gave us via the contact form
			foreach(array_keys($userpasswd->properties['var_index']) as $property) {
				if(!isset($this->obj[$property])) {
					// this if case is included because Username <> contact_name,
					// resolved outside foreach-loop
					continue;
				}
				$userpasswd->obj[$property] = $this->obj[$property];
				unset($this->obj[$property]);
			}
			if(isset($this->obj['contact_name']) && $this->obj['contact_name']) {
				$userpasswd->obj['username'] = $this->obj['contact_name'];
			}
		}
		else {
			foreach(array_keys($userpasswd->properties['var_index']) as $property) {
				if(isset($this->obj[$property])) {
					unset($this->obj[$property]);
				}
			}
			return parent::save_object($user);
		}

		if(!$this->validate_object()) {
			return false;
		}

		if(!$userpasswd->validate_object()) {
			$this->validation_error_list = $userpasswd->validation_error_list;
			return false;
		}

		if(!parent::save_object($user)) {
			return false;
		}

		return  $userpasswd->save_object($user);
	}

	// @todo make sure that the merged user properties are removed before banishing
	function banish_object($user=true) {
		return parent::banish_object($user);
	}
}
