<?php
require_once(dirname(__FILE__) . '/../include/webconfig.inc.php');
require_once(dirname(__FILE__) . '/../include/timeperiods.php');
require_once(dirname(__FILE__) . '/../include/import.inc.php');
require_once(dirname(__FILE__) . '/../include/probe.inc.php');

/**
 * The gui_class is responsible for drawing all the various elements
 * of the page.
 */
class gui_class
{
	private $form_name = false;
	private $html_started = false;

	/**
	 * Constructor. Responsible for validating that the user is
	 * authorized to use the GUI and to make sure this is no
	 * poller system in a distributed setup.
	 */
	function gui_class()
	{
		if (!validate_user()) {
			# Unauthorized ...
			
			$msg = "<p>Access denied</p>\n" .
			"<p class=\"note\">It appears as though you aren't authorized to access the configuration interface.</p>
			<p>Note that at least one of your groups must be authorized for both <strong>'configuration_information'</strong>
			and <strong>'system_commands'</strong> to be able change any configuration.</p>";
			if (PHP_SAPI !== 'cli') {
				echo $msg;
			} else {
				print strip_tags($msg);
			}
			$this->html_end();
		}

		if (is_poller()) {
			# This is a poller in a distributed setup.
			if(!class_exists('Changelog')) {
				require dirname(__FILE__).'/changelog_class.php';
			}
			$this->html_start();
			echo "<p><strong>This is a poller system!</strong></p>\n";
			echo "It appears this is a poller system in a distributed setup.\n";
			echo "<br /><br />\n";
			echo "No configuration is supposed to be made on a poller system.\n";
			$this->html_end();
		}
	}

	/**
	 * Import configuration from files to database
	 * @param $reason Reason why import was run
	 * @param $user The user running the import
	 */
	function run_import($reason, $user = false)
	{
		global $import_errors, $import_warnings;
		global $SUPPORT;

		echo "<p><strong>$reason<br />\n";
		$success = import_config($user);
		$errors = $this->print_problem_array('Import errors', $import_errors);

		$this->print_problem_array('Import warnings', $import_warnings);
		if (!$success || $errors) {
			echo "If you have manually made modifications to the configuration, " .
				"look it over to make sure you haven't done anything wrong with it.<br />\n";
			echo "You can run the command<pre>\nservice monitor configtest</pre> to " .
				"verify that the configuration you have created is valid. If the " .
				"configuration is valid and you still see this message " .
				"you have most likely found a template that is not currently in " .
				"use that references an object that doesn't exist.<br />" .
				"If that is not the case, then this is a bug.<br />" .
				"Please contact $SUPPORT so that we can help you fix this problem.<br />\n";

			echo "<p>Irrecoverable errors encountered while importing. Bailing out<br />\n";
			$this->html_end();
		}
		echo "Configuration has been imported, overwriting database changes.</strong><br />\n";
	}

	/**
	 * Import the configuration if there are changes in the files
	 * @param $user The user running the import
	 */
	function import_if_changes($user = false)
	{
		if (!get_last_import_time()) {
			return $this->run_import('No imported configuration exists', $user);
		}
		if (!changes_since_last_import())
			return true;

		$this->run_import('Config files have changed since last import', $user);
	}

	/**
	 * Print HTML header
	 * @param $subtitle string: Page subtitle
	 */
	function html_start($subtitle = false)
	{
		global $DEBUG, $Version;

		if($this->html_started)
		  return true;

		$this->html_started = true;

		$user = get_user();
		$date_str = date('d M h:i:s T Y');

		if(PHP_SAPI !== 'cli') {
			header('Cache-Control: no-cache');
			header('Content-Type: text/html; charset=utf-8');
		}
		if ($DEBUG) {
			require_once(dirname(__FILE__).'/../vendor/FirePHPCore/fb.php');
			ob_start();
		}
		require_once( dirname(__FILE__) . '/../templates/html_start.php');
	}

	/**
	 * Print HTML footer
	 */
	function html_end($exit_code=0)
	{
		global $DEBUG;
		require_once( dirname(__FILE__).'/../templates/html_end.php');
		// may fail, as we only turn on buffering in html_start
		// which isn't executed for e.g. tests
		if ($DEBUG)
			@ob_end_flush();
		exit($exit_code);
	}

	/**
	 * Remove save prompt from page top
	 */
	function js_remove_save_warning()
	{
		?><script type="text/javascript">
	    <!--
	    $(".config-warning").remove();
	    //-->
	    </script><?php
	}

	/**
	 * Create a form
	 * @param $name Form name
	 * @param $method Form method
	 * @param $action Form action
	 */
	function form_start($name = 'foo_form', $method = 'post', $action = false)
	{
		global $_SERVER, $input;

		$this->form_name = $name;

		$action = $action ? $action : $_SERVER['PHP_SELF'];
		echo "<form method=\"$method\" action=\"$action\" name=\"".$name."\" id=\"$name\" onsubmit=\"return check_required_objects(this)";
		if ($input->obj_type == 'timeperiod' && $name == 'foobar_form')
			echo " && timeperiod_add_custom_vars(this)";
		echo ";\">\n";
	}

	/**
	 * Close a form
	 * @param $return Unused
	 */
	function form_end($return = false)
	{
		global $DEBUG;

		if ($DEBUG && !$this->form_name)
		  echo "Form closed when none started!<br />\n";

		$this->form_name = false;
		echo "\n</form>\n";
	}

	/**
	 * Get messages stored in session, deletes them after retreival
	 *
	 * @return array of strings
	 */
	function get_flashed_messages() {
		if(!session_id() || !isset($_SESSION['nacoma_flash'])) {
			return array();
		}
		$messages = $_SESSION['nacoma_flash'];
		unset($_SESSION['nacoma_flash']);
		return $messages;
	}

	/**
	 * Draw a form submit button
	 *
	 * @param $text The text on the button, or FALSE for default
	 * @param $name The form name of the button, or FALSE for default
	 * @param $skinny If false, the button will take up a comfortable amount of space. If true, it will be narrower
	 * @param $extra An array of HTML attribute key-vals to append/replace the automatic ones
	 */
	function submit_button($text = false, $name = false, $skinny = false, $extra = false)
	{
		if (!$text)
			$text = 'Submit';
		if (!$name)
			$name = 'action';
		$output = array(
			'type' => 'submit',
			'name' => $name,
			'value' => $text,
			'title' => $text,
			'onclick' => 'loopElements(this.form);',
			'class' => 'object-submit'
		);

		if (is_array($extra))
			$output = array_merge($output, $extra);

		// for hysterical reasons, the value of the action form element will be
		// randomly overwritten, which looks ugly, so we hide that uglyness
		if ($output['name'] == 'action') {
			$this->hidden_var($output['name'], $output['value']);
			$output['name'] = 'action_dummy';
		}

		$attributes = '';
		foreach ($output as $key => $val) {
			$attributes .= " $key=\"".htmlspecialchars($val).'"';
		}

		if (!$skinny)
			echo '<div class="magic">';
		echo "<input".$attributes." />\n";
		if (!$skinny)
			echo '</div>';
	}

	/**
	 * Add a hidden form variable
	 * @param $name Name of the variable
	 * @param $value Value of the variable
	 * @param $return Unused
	 */
	function hidden_var($name, $value, $return = false)
	{
		$name = str_replace('"', '&quot;', $name);
		$value = str_replace('"', '&quot;', $value);
		echo "<input type=\"hidden\" name=\"$name\" value=\"$value\" />\n";
	}

	/**
	 * Print the current object on the webpage
	 * @param $obj class_object A class related to a specific object type
	 */
	function draw_object($obj)
	{
		$obj->prepare_draw();
		$this->print_related($obj);
		$this->pretty_print_one($obj);
	}

	/**
	 * Create a filtered selection inside a webform
	 * @param $values Array of values in the form
	 * @param $dflt The default selected value
	 * @param $select_name The form name of this variable
	 */
	function make_filtered_selection($values, $dflt = '', $select_name = 'selection')
	{
		if (!$this->form_name) {
			bug("Filtered selection only work inside forms");
			return;
		}
		require_once('widgets/select.php');
		select_widget::addFilterlistButtons(sizeof($values));
		select_widget::make_selection($values, $dflt, $select_name);
		select_widget::addFilterlist($select_name, sizeof($values));
	}

	/**
	 * Generic function to create selection lists from any type
	 * of object that also has a group-type (hosts, services and
	 * contacts).
	 * The input_class understands the format we create here and
	 * can parse them correctly, which aids the various programs
	 * rather nicely.
	 * @param $obj_type Object type
	 * @param $exclude Objects to exclude from listing
	 * @param $target_type Ultimate target type
	 */
	function make_multi_obj_selection($obj_type, $exclude = array(), $target_type = false)
	{
		# allow single-item exclusion (the most common kind)
		# but special-case 'new' for when we're for example
		# propagating variables from an object that doesn't exist
		if (!empty($exclude) && !is_array($exclude)) {
			if ($exclude === 'new')
				$exclude = false;
			else
				$exclude = array($exclude);
		}

		if (!$target_type)
			$target_type = $obj_type;

		switch ($obj_type) {
		 case 'host': case 'service': case 'contact':
			$group_type = $obj_type . 'group';
			$query = "SELECT id, " . $group_type . "_name, alias FROM $group_type";
			$query .= " WHERE id IN (SELECT DISTINCT " . $group_type .
				" FROM " . $obj_type . "_$group_type" .
				" WHERE id ".get_exclude_pending_string($group_type);

			# for the group-query, we exclude groups that have only those
			# objects listed in exclude as members
			if (!empty($exclude)) {
				$query .= " AND $obj_type NOT IN (" . join(", ", $exclude) . ")";
			}

			$query .= ")";

			$result = pdo_exec_query($query);

			$group_list = array();
			foreach ($result->fetchAll(PDO::FETCH_NUM) as $row) {
				if (object_class::can_user_read($group_type, $row[0]))
					$group_list[$row[0]] = "$row[1] - $row[2]";
			}
					unset($result);
			if (!empty($group_list)) {
				echo "&nbsp;<strong>" . ucfirst($target_type) . "s";
				if ($target_type !== $obj_type) {
					echo " assigned to " . $obj_type . "s";
				}
				echo " in the following ". $group_type . "s</strong>\n";
				$this->make_multi_selection($group_list, false, 'selected_' . $obj_type . '_groups');
				echo "<br />\n";
			}
			break;
		 default:
			 break;
		}

		switch ($obj_type) {
		 # object types with both *_name and alias
		 case 'contact': case 'contactgroup': case 'host': case 'hostgroup':
		 case 'servicegroup': case 'timeperiod':
			$query = "SELECT id, " . $obj_type . "_name, alias FROM $obj_type " .
				"WHERE register != 0 AND id ".get_exclude_pending_string($obj_type)." ORDER BY " . $obj_type . "_name";
			$result = pdo_exec_query($query);
			foreach (($result->fetchAll(PDO::FETCH_NUM)) as $row) {
				if (object_class::can_user_read($obj_type, $row[0]))
					$obj_list[$row[0]] = "$row[1] - $row[2]";
			}
			unset($result);
			break;
		 default:
			$obj_list = get_objects_for_user($obj_type);
			break;
		}

		if (is_array($exclude)) foreach ($exclude as $id) {
			$this->hidden_var('select_exclude['.$target_type.'][]', $id);
			if (isset($obj_list[$id]))
				unset($obj_list[$id]);
		}

		echo "&nbsp;<strong>";
		if ($target_type !== $obj_type)
			echo ucfirst($target_type) . "s assigned to the following ";
		echo ucfirst($obj_type) . "s</strong>:\n";
		$this->make_multi_selection($obj_list, false, 'selected_' . $obj_type . 's');
	}

	/**
	 * Simple wrapper around make_multi_obj_selection()
	 * @param $exclude array: Hosts to exclude
	 */
	function make_multi_host_selection($exclude = array())
	{
		$this->make_multi_obj_selection('host', $exclude);
	}

	/**
	 * Simple wrapper around make_multi_obj_selection()
	 * @param $exclude array: Services to exclude
	 */
	function make_multi_service_selection($exclude = array())
	{
		$this->hidden_var('selection_target', 'service');
		$this->make_multi_obj_selection('host', array(), 'service');
		echo "<br />\n";
		$this->make_multi_obj_selection('service', $exclude);
	}

	/**
	 * Prints an array of entries, using $subject as the table
	 * header and then printing one line of text per row.
	 * If $ary is empty, this function does nothing.
	 *
	 * @return The number of entries printed.
	 */
	function print_problem_array($subject, $ary)
	{
		if (empty($ary))
			return 0;

		echo "<table class=\"zebra\" summary=\"\"><tr class='header'><td>$subject</td></tr>\n";
		$i = 0;
		foreach ($ary as $problem) {
			$i++;
			echo "<tr><td class='List'><pre>$problem</pre></td></tr>\n";
		}
		echo "</table>\n";

		return $i;
	}

	/**
	 * Print a list of object validation errors
	 * @param $obj A Nagios-object-specific php object
	 * @param $ary The validation errors, if not set in $obj
	 */
	function print_validation_error_table($obj, $ary = false)
	{
		if ($ary === false && empty($obj->validation_error_list)) {
			bug("print_validation_error_table() called, but object has no validation_error_list");
			return;
		}

		$erray = $ary === false ? $obj->validation_error_list : $ary;
		echo "<table summary=\"\" style=\"width: 100%;border-bottom: 1px solid #dcdcdc; border-collapse: collapse; border-spacing: 0px\" class=\"zebra\"><tr class='header'><td style='padding-left: 7px;'>Variable</td><td style='padding-left:7px;'>Reason</td></tr>\n";
		foreach ($erray as $v => $reason) {
			echo "<tr>
				<td style='padding: 2px 7px; border-right: 1px solid #dcdcdc'>$v</td>
				<td style='padding: 2px 7px;'>$reason</td>
			</tr>\n";
		}
		echo "</table>\n";
	}

	/**
	 * Print links and info related to object type. REWRITE ME PLEASE!!!
	 * @param $obj A Nagios-object-specific php class object
	 */
	function print_related($obj)
	{
		global $Supported_Object_Types;
		global $show_logserver_scan;
		$host 		= false;
		$service 	= false;
		$related_items = false;
		$obj_types = $Supported_Object_Types;
		$obj_pages = false;
		if (!$obj){
			// Top level info text goes here.
			echo "<br /><table class=\"MainMenuTable\" summary=\"Main menu table\" summary=\"\">\n";

			$Menu_Item = false;
			$Menu_Item =
			  array('templates' => 'Templates',
					'hostgroup' => 'Host Groups',
					'servicegroup' => 'Service Groups',
					'contact' => 'Contacts',
					'contactgroup' => 'Contact Groups',
					'command' => 'Check Commands',
					'timeperiod' => 'Time Periods');

			// removed this	for now
			/*
			if(is_super_user()) {
				$Menu_Item['access'] = 'Access Rights';
			}
			*/
			foreach($Menu_Item as $type => $label) {
				echo "<tr><td class=\"MainMenuItem\"><a href=\"edit.php?obj_type=" . $type . "\" title=\"" . $label . " Configuration\"	>" .
				  $label . "</a></td></tr>\n";
			}

			echo "</table>\n";
			exit;
		}

		if ($obj->type==='servicedependency' ||  $obj->type==='serviceescalation'){
			require_once('service_class.php');
			$svc = new service_class($obj->master_obj_id);
			$gp_id = $svc->master_obj_id;
			$gp_type = $svc->master_obj_type;
			$gp = $this->get_index_property($svc->master_obj_type, $svc->master_obj_id);
		}

		if (isset($obj->master_obj_candidates['host']) && isset($obj->master_obj_candidates['hostgroup']))
			$parent_icon = $obj->master_obj_type == 'host' ? 'host-configuration.png' : 'host-groups.png';


		switch ($obj->type){
			case 'contact': case 'contact_template':
				// contact
				$related_items['contact'][] = '<img src="images/icons/local-users.png" alt="" /> <a href="edit_special.php/access">Local Users</a>';
				$related_items['contact'][] = '<img src="images/icons/contact-groups.png" alt="" /> <a href="edit.php?obj_type=contactgroup">Contact Groups</a>';
				$related_items['contact'][] = '<img src="images/icons/time-periods.png" alt="" /> <a href="edit.php?obj_type=timeperiod">Timeperiods</a>';
				if (!$obj->is_template) {
					$related_items['contact'][] = '<img src="images/icons/templates.png" alt="" /> <a href="edit.php?obj_type=contact_template">Contact Templates</a>';
				}
				break;
			case 'hostdependency':
				if ($obj->master_obj_id) $related_items['hostdependency'][] = '<img src="images/icons/'.$parent_icon.'" alt="" /> <a href="edit.php?obj_type='.$obj->master_obj_type.'&amp;obj_id='.$obj->master_obj_id.'">'.ucfirst($obj->master_obj_type).' configuration: <b>'.$this->get_index_property($obj->master_obj_type, $obj->master_obj_id).'</b></a>';
				break;
			case 'servicedependency':
				if (isset($gp_id))
					$related_items['servicedependency'][] = '<img src="images/icons/services.png" alt="" /> <a href="edit.php?obj_type=service&amp;master_obj_id='.$gp_id.'&amp;master_obj_type='.$gp_type.'">Services for '.$gp_type.': <b>'. $gp .'</b></a>';
				elseif ($gp!='')
					$related_items['servicedependency'][] = '<img src="images/icons/services.png" alt="" /> <a href="edit.php?obj_type=service">Services for '.$gp_type.': <b>'. $gp .'</b></a>';
				break;
			case 'serviceescalation':
				if (isset($gp_id))
					$related_items['serviceescalation'][] = '<img src="images/icons/services.png" alt="" /> <a href="edit.php?obj_type=service&amp;master_obj_id='.$gp_id .'&amp;master_obj_type='.$gp_type.'">Services for '.$gp_type.' <b>'. $gp .'</b></a>';
				$related_items['serviceescalation'][] = '<img src="images/icons/contact-groups.png" alt="" /> <a href="edit.php?obj_type=contactgroup">Contact Groups</a>';
				$related_items['serviceescalation'][] = '<img src="images/icons/time-periods.png" alt="" /> <a href="edit.php?obj_type=timeperiod">Time Periods</a>';
				break;
			case 'hostescalation':
				if ($obj->master_obj_id)
					$related_items['hostescalation'][] = '<img src="images/icons/'.$parent_icon.'" alt="" /> <a href="edit.php?obj_type='.$obj->master_obj_type.'&amp;obj_id='.$obj->master_obj_id.'">'.ucfirst($obj->master_obj_type).' Configuration: <b>'.$this->get_index_property($obj->master_obj_type, $obj->master_obj_id).'</b></a>';
				$related_items['hostescalation'][] = '<img src="images/icons/contact-groups.png" alt="" /> <a href="edit.php?obj_type=contactgroup">Contact Groups</a>';
				$related_items['hostescalation'][] = '<img src="images/icons/time-periods.png" alt="" /> <a href="edit.php?obj_type=timeperiod">Time Periods</a>';
				break;
			case 'contactgroup':
				$related_items['contactgroup'][] = '<img src="images/icons/contacts.png" alt="" /> <a href="edit.php?obj_type=contact">Contacts</a>';
				break;
			case 'host': case 'host_template':
				if (!$obj->is_template && (int)$obj->id) {
					$related_items['host'][] = '<img src="images/icons/scan-host.png" alt="" /> <a href="host_wizard.php?action=scan&amp;obj_id='.$obj->id.'" class="scan network">Scan host for network services</a>';
					$related_items['host'][] = '<img src="images/icons/scan-host.png" alt="" /> <a href="host_wizard.php?action=snmp_scan&amp;obj_id='.$obj->id.'" class="scan snmp">Scan host for SNMP interfaces</a>';
					$related_items['host'][] = '<img src="images/icons/scan-host.png" alt="" /> <a href="host_wizard.php?action=choose_win_check_type&amp;obj_id='.$obj->id.'" class="scan windows">Scan host for Windows Services</a>';
					if ($show_logserver_scan)
						$related_items['host'][] = '<img src="images/icons/scan-host.png" alt="" /> <a href="host_wizard.php?action=check_logserver_filters&amp;obj_id='.$obj->id.'">Scan host for Logserver filters</a>';
					$related_items['host'][] = '<img src="images/icons/services.png" alt="" /> <a href="edit.php?obj_type=service&amp;host_id='.$obj->id.'">Services for host <b>' . $this->get_index_property('host', $obj->id) . '</b></a>';
					$related_items['host'][] = '<img src="images/icons/host-configuration.png" alt="" /> <a title="View status information for this host (Opens in new window)" href="/monitor/index.php/extinfo/details/host/'.$this->get_index_property('host', $obj->id).'" target="_blank">Status information</a>';
				}
				if (!$obj->is_template) {
					$related_items['host'][] = '<img src="images/icons/templates.png" alt="" /> <a href="edit.php?obj_type=host_template">Host Templates</a>';
				}
				$related_items['host'][] = '<img src="images/icons/commands.png" alt="" /> <a href="edit.php?obj_type=command">Check Commands</a>';
				$related_items['host'][] = '<img src="images/icons/contact-groups.png" alt="" /> <a href="edit.php?obj_type=contactgroup">Contact Groups</a>';
				$related_items['host'][] = '<img src="images/icons/time-periods.png" alt="" /> <a href="edit.php?obj_type=timeperiod">Time Periods</a>';
				if (!$obj->is_template) {
					$related_items['host'][] = '<img src="images/icons/new-host.png" alt="" /> <a href="host_wizard.php">Add new host</a>';
				}
				break;
			case 'service': case 'service_template':
				if (!$obj->is_template && $obj->master_obj_id) {
					if ($obj->master_obj_type === 'host') {
						$related_items['service'][] = '<img src="images/icons/scan-host.png" alt="" /> <a href="host_wizard.php?action=scan&amp;obj_id='.$obj->master_obj_id.'" class="scan network">Scan host for network services</a>';
						$related_items['service'][] = '<img src="images/icons/scan-host.png" alt="" /> <a href="host_wizard.php?action=snmp_scan&amp;obj_id='.$obj->master_obj_id.'" class="scan snmp">Scan host for SNMP interfaces</a>';
						$related_items['service'][] = '<img src="images/icons/scan-host.png" alt="" /> <a href="host_wizard.php?action=choose_win_check_type&amp;obj_id='.$obj->master_obj_id.'" class="scan windows">Scan host for Windows Services</a>';
						if ($show_logserver_scan)
							$related_items['service'][] = '<img src="images/icons/scan-host.png" alt="" /> <a href="host_wizard.php?action=check_logserver_filters&amp;obj_id='.$obj->master_obj_id.'">Scan host for Logserver filters</a>';
					}
					$related_items['service'][] = '<img src="images/icons/'.$parent_icon.'" alt="" /> <a href="edit.php?obj_type='.$obj->master_obj_type.'&amp;obj_id='. $obj->master_obj_id.'">'.ucfirst($obj->master_obj_type).' configuration: <b>'.$this->get_index_property($obj->master_obj_type, $obj->master_obj_id) .'</b></a>';
					if ((int)$obj->id)
						$related_items['service'][] = '<img src="images/icons/host-configuration.png" alt="" /> <a title="View status information for this service (Opens in new window)" href="/monitor/index.php/extinfo/details/service/'.$this->get_index_property($obj->master_obj_type, $obj->master_obj_id).'?service='.urlencode($obj->obj['service_description']).'" target="_blank">Status information</a>';
				}
				if (!$obj->is_template) {
					$related_items['service'][] = '<img src="images/icons/templates.png" alt="" /> <a href="edit.php?obj_type=service_template">Service Templates</a>';
				}
				$related_items['service'][] = '<img src="images/icons/commands.png" alt="" /> <a href="edit.php?obj_type=command">Check Commands</a>';
				$related_items['service'][] = '<img src="images/icons/contact-groups.png" alt="" /> <a href="edit.php?obj_type=contactgroup">Contact Groups</a>';
				$related_items['service'][] = '<img src="images/icons/time-periods.png" alt="" /> <a href="edit.php?obj_type=timeperiod">Time Periods</a>';
				$related_items['service'][] = '<img src="images/icons/service-groups.png" alt="" /> <a href="edit.php?obj_type=servicegroup">Service Groups</a>';
				$related_items['service'][] = '<img src="images/icons/services.png" alt="" /> <a href="edit_special.php/copy_slaves?master_obj_id='.$obj->master_obj_id.'&amp;master_obj_type='.$obj->master_obj_type.'&amp;obj_id='.$obj->id.'&amp;obj_type='.$obj->type.'">Clone Service(s)</a>';
				break;
			case 'command':
				if ( file_exists('/opt/plugins/metadata/'))
					$related_items['command'][] = '<img src="images/icons/check-command-import.png" alt="" /> <a href="metadata.php">Check Command Import</a>';
				break;
			case 'hostgroup':
				if ((int)$obj->id)
					$related_items['hostgroup'][] = '<img src="images/icons/services.png" alt="" /> <a href="edit.php?obj_type=service&amp;master_obj_id='.$obj->id.'&amp;master_obj_type=hostgroup">Services for hostgroup <b>' . $obj->get_object_name() . '</b></a>';
				break;
		}

		// Remove FILE object
		array_pop($obj_types);
		sort($obj_types);
		for ($i=0;$i<sizeof($obj_types);$i++){
			$obj_pages[$obj_types[$i]] = $obj_types[$i];
		}
		$obj_pages['template'] = 'template';

		if (($obj->type == 'service' || $obj->type == 'serviceescalation' || $obj->type == 'servicedependency'
			 || $obj->type == 'hostdependency' || $obj->type == 'hostescalation')&& !$obj->master_obj_id)
		{
			echo '</div></div>';
		}

		if (in_array($obj->type, $Supported_Object_Types))
			$related_items[$obj->db_table][] = '<img alt="" src="images/icons/delete.png" /><a href="edit_special.php/bulk_delete?obj_type='.$obj->type.'">Bulk delete objects</a>';
		if (count($related_items[$obj->db_table])) {
			echo '<div id="related">';
			$obj_id_link = "";
			if ($obj->id) {
				$obj_id_link = "&amp;master_obj=" . $obj->id;
			}
			$onchange = "set_page_location('$obj->type', this, '$obj_id_link');";
			$this->make_drop_down($obj_pages, $obj->type, 'page_selection', 'xlarge', $onchange);
			echo "<br /><strong>Related items</strong><br />\n";
			echo '<ul class="config">';
			for ($i=0;$i<sizeof($related_items[$obj->db_table]);$i++)
				echo '<li>'.$related_items[$obj->db_table][$i] . "</li>\n";
			echo '</ul>';
			echo '</div>'; // #related
		}
		if (($obj->type == 'service' || $obj->type == 'serviceescalation' || $obj->type == 'servicedependency'
			|| $obj->type == 'hostdependency' || $obj->type == 'hostescalation')&& !$obj->master_obj_id) {
			echo '<div>';
		}
	}

	/**
	 * Create the link "Services for {host,hostgroup}"
	 * @return string: The link, complete with image and all
	 */
	function print_services_link($obj)
	{
		if (($obj->type !== 'host' && $obj->type !== 'hostgroup') || !$obj->id || $obj->id === 'new')
			return;

		return "<img src=\"images/icons/arrow.gif\" alt=\"\" /> &nbsp;<a href='edit.php?obj_type=service&amp;master_obj_id=$obj->id&amp;master_obj_type=$obj->type'>Services for {$obj->type} <b>{$obj->get_object_name()}</b></a>";
	}

	/**
	 * Create drop-down (select) with the option of specifying
	 * classname and onchange action
	 * @param $objects array: Objects to list
	 * @param $dflt string: Default object
	 * @param $selection_name string: Name of the selection
	 * @param $classname string: css class name
	 * @param $onchange javascript gunk to add to form
	 */
	function make_drop_down($objects=false, $dflt='', $selection_name='', $classname='xlarge', $onchange = '')
	{
		$dflt 			= addslashes(trim($dflt));
		$selection_name = addslashes(trim($selection_name));
		$classname 		= addslashes(trim($classname));
		$onchange		= trim($onchange);
		echo "<select name=\"".$selection_name."\" class=\"".$classname."\"";
		echo $onchange!='' ? " onchange=\"".$onchange."\">" : ">";
		foreach($objects as $item){
			echo "<option value=\"".$item."\"";
			if ($dflt==$item){
				echo " selected=\"selected\"";
			}
			echo ">".$item."</option>";
		}
		echo "</select>";
	}

	/**
	 * Get an object from $Object_Index_Table
	 * @param $type string: Nagios-style object type
	 * @param $id int: Id of the object
	 * @return string: The object name
	 */
	function get_index_property($type='', $id=0)
	{
		$id 	= (int)$id;
		$type 	= addslashes( trim($type) );

		if (!$id || $type===''){
			return 'unknown';
		}
		$ret = get_object_index_table($type);
		return isset($ret[$id]) ? $ret[$id] : false;
	}

	/**
	 * Print custom variables for one object
	 * @param $obj A Nagios-specific php object class
	 */
	function pretty_print_custom_vars($obj)
	{
		// Warning! These HTML selectors needs to correspond
		// to the similar code in common.js
		echo "<tr class='custom'>
			<td class='HelpList custom_vars_helpfield'><a href='#' title='Click to view help on custom_vars' tabindex='-1' onclick=\"ShowContent('helpContent', 'custom_vars'); return false;\"><img src='images/icons/shield-help.png' alt='Help' title='Help' /></a></td>
			<td></td>
			<td>Custom variable:</td>
			<td>Value:</td>
		</tr>";
		if (!empty($obj->custom_vars)) {
			$i = 0;
			$base_var_name = 'custom_vars[' . $obj->type . '][' . $obj->id . ']';
			$line_start = '<td><input type="text" name="' . $base_var_name;
			foreach ($obj->custom_vars as $k => $v) {
				echo "<tr class=\"custom\">\n";
				$k = htmlspecialchars($k);
				$v = htmlspecialchars($v);
				echo '<td></td>';
				echo '<td>'.$this->print_propagate_checkbox($k).'</td>';
				echo $line_start . "[$i][key]\" value=\"$k\"></td>\n";
				echo $line_start . "[$i][value]\" value=\"$v\">\n";
				echo '<input type="button" class="remove_custom" value="Remove" />';

				$i++;
				echo "</td></tr>\n";
			}
		}
		echo '<tr class="custom"><td></td><td></td><td></td><td><input type="button" id="new_custom" value="Add custom variable" /></td></tr>';
	}

	/**
	 * Returns a checkbox used for selecting variables to propagate
	 * @param $token The name of the variable
	 */
	function print_propagate_checkbox($token)
	{
		if (empty($token))
			return;
		return "<input type='checkbox' class=\"propagate_box\" title=\"Check to propagate this property\" id=\"prop_vars[$token]\" name='prop_vars[$token]' />\n";
	}

	/**
	 * Return html for drop-down boxes to choose node to test from.
	 *
	 * Warning!
	 *
	 * This is slow. We do not want to call this unless we must
	 */
	private function get_node_html() {
		$nodes = get_merlin_nodes();
		$nodehtml = false;
		if (count($nodes) > 1) {
			$nodehtml = '<span class="extra_test"> on <select id="cmdTestNode" name="cmdTestNode">';
			foreach ($nodes as $node) {
				if ($node === 'This server')
					$nodehtml .= '<option value="" selected="selected">'.$node.'</option>';
				else
					$nodehtml .= '<option value="'.$node.'">'.$node.'</option>';
			}
			$nodehtml .= '</select></span>';
		}
		return $nodehtml;
	}

	/**
	 * Print form for each object
	 * @param $obj Nagios-specific php object class
	 */
	function pretty_print_one($obj)
	{
		// get the object properties
		$special = $obj->properties['special'];
		$obj_index = $obj->properties['var_index'];
		$req_vars = $obj->properties['required'];

		// Create array of objects for required fields and data type
		if (!empty($req_vars)) { ?>
			<script type="text/javascript">
				monitor_required_objects = new Array();
			<?php	$a = 0;
					foreach ($req_vars as $req) {
						if (!array_key_exists($req, $obj_index) || (preg_match('/_options$/', $req, $discard)) && $obj_index[$req]=='checkbox') continue;?>
						monitor_required_objects[<?php echo $a ?>] = new required_vars('<?php echo $req ?>', '<?php echo $obj_index[$req] ?>');
					<?php
						if (isset($obj->properties['req_alt'][$req])) { ?>
						monitor_required_objects[<?php echo $a ?>].req_alt = '<?php echo $obj->properties['req_alt'][$req]; ?>';
						monitor_required_objects[<?php echo $a ?>].alt_type = '<?php echo $obj_index[$obj->properties['req_alt'][$req]]; ?>';
						<?php }
						$a++;
					}?>
			</script><?php
		}
		// Oh IE7, you so silly...
		echo "<div style='float: none; clear:both'></div>";
		// Create the All Surrounding object table
		echo "<div class=\"object-table\">\n" .
		  "<table class=\"ObjTable max zebra\" summary=\"Object Table\">\n <tr><td colspan=\"4\" style=\"padding: 0; border: 0;\">\n";

		$this->draw_command_bar($obj);
		echo "</td></tr>\n";

		/* Ok. Command bar is completed, so we start printing the
		 * actual variables */
		$this->hidden_var('tmp_obj_id', $obj->id);
		$this->hidden_var('obj_type', $obj->type);
		if (isset($obj->clone_id)) {
			$this->hidden_var('clone_id', $obj->clone_id);
		}

		foreach($obj_index as $token => $v_type) {
			if (!isset($obj->obj[$token]))
				continue;

			$value = $obj->obj[$token];

			if ($v_type != 'hidden' && $v_type != 'norender') {
				echo "<tr class=\"" . str_replace(' ', '_', $token) . "_row\">\n \n";
				echo " <td class=\"HelpList " . str_replace(' ', '_', $token) . "_helpfield\"><a href=\"#\" title=\"Click to view help on ".$token."\" tabindex='-1' onclick=\"ShowContent('helpContent', '".$token."'); return false;\"><img src=\"images/icons/shield-help.png\" alt=\"Help\" title=\"Help\" /></a></td>";
				echo "<td>";
				# print the checkbox used to select variables for
				# propagation, but we don't do it for commands or
				# variables used to uniquely identify an object
				if ($obj->type !== 'command' && $token != $obj->type . '_name' && !empty($obj->id)) {
					echo $this->print_propagate_checkbox($token);
				}
				echo "  </td>\n";
				$token_label = isset($obj->properties['label'][$token]) ? $obj->properties['label'][$token] : $token;
				echo " <td class=\"VarList " . str_replace(' ', '_', $token) . "_namefield\">$token_label\n</td>\n";

				echo "  <td class=\"" . str_replace(' ', '_', $token) . "_valuefield\">";
			}

			$this->insert_widget($v_type, $obj, $token, $value);

			if ($v_type != 'hidden' && $v_type != 'norender') {
				echo "  </td>";
				echo "  \n</tr>";
			}
		}

		switch ($obj->type) {
		 case 'timeperiod':
			timeperiod_exceptions_print($obj, $this);
			break;
		 case 'host': case 'host_template':
		 case 'service': case 'service_template':
		 case 'contact': case 'contact_template':
			$this->pretty_print_custom_vars($obj);
			break;
		}


		if ($obj->type==='command' && $obj->id!=='new' || ( $obj->id=='new'  && isset($obj->obj['command_line']) && $obj->obj['command_line'] != '')) { // test check command
			$nodehtml = $this->get_node_html();
			echo "<tr><td colspan=\"4\" class=\"testButton \"><input type=\"button\" id=\"cmdTestButton\" class=\"button test-this-command\" onclick=\"printCmdForm(this.form, 'command_line');\" title=\"Click to test this command\" value=\"Test this command\"><div id=\"commandTest\" style=\"display: none;\">$nodehtml</div></td></tr>\n";
		}
		if ($obj->type==='service') { // test service
			$nodehtml = $this->get_node_html();
			echo "<tr><td colspan=\"4\" class=\"testButton \"><input type=\"button\" id=\"cmdTestButton\" class=\"button test-this-service\" onclick=\"testService(this.form);\" title=\"Click to test service with the values entered above\" value=\"Test this service\">$nodehtml<div id=\"commandTest\"></div></td></tr>\n";
		}
		if ($obj->type==='host') {
			$nodehtml = $this->get_node_html();
			if(isset($obj->obj['check_command'])) {
				echo "<tr><td colspan=\"4\" class=\"testButton\" style='border: 1px solid #cdcddc;border-bottom: 0px'><input type=\"button\" id=\"cmdTestButton\" class=\"button test-this-host\" onclick=\"testService(this.form, 'tmp_obj_id');\" title=\"Click to test service with the values entered above\" value=\"Test this host\">$nodehtml<div id=\"commandTest\"></div></td></tr>\n";
			} else {
				echo "<tr><td colspan=\"4\" class=\"testButton\" style='border: 1px solid #cdcddc;border-bottom: 0px'><input type=\"button\" disabled='disabled' id=\"cmdTestButton\" class=\"button test-this-host\" value=\"Test this host\">$nodehtml<div id=\"commandTest\"></div>Add a check command to be able to test this host.".(!isset($_GET['action']) || $_GET['action'] != 'expand' ? "Also, make sure you are on the 'advanced' tab." : null)."</td></tr>\n";
			}
		}
		echo "</table>\n";
		echo '</div>';
	}

	/**
	 * Include and render the correct widget for the specified token
	 */
	function insert_widget($name, $obj, $token, $value) {
		require_once('widgets/'.$name.'.php');
		$class = $name.'_widget';
		$widget = new $class();
		$widget->render($obj, $token, $value);
	}

	/**
	 * Get variables is shown on page
	 *
	 * @param $obj A Nagios-specific php object class
	 * @return array: key => value indexed array
	 */
	function get_page_vars($obj)
	{
		$variables = array();
		// get the object properties
		$special 	= $obj->properties['special'];
		$v_index 	= $obj->properties['var_index'];
		$req_vars 	= $obj->properties['required'];

		foreach($v_index as $token => $v_type) {
			if(!isset($obj->obj[$token]))
			  continue;
			$value = $obj->obj[$token];
			/* hidden variables are dealt with here, to save us
			 * exception handling in the middle of all if .. elseif .. */
			if($v_type === 'hidden' ||
			   ($token === 'FILE' &&
				$obj->id !== 'new' &&
				$obj->type !== 'new_host' &&
				$obj->type !== 'new_service' &&
				(empty($_REQUEST['action']) ||
				 $_REQUEST['action'] !== 'expand')))
			{
				continue;
			}
			$variables[] = $token;
		}
		return $variables;
	}

	/**
	 * Generate help_link
	 * @param $obj A Nagios-specific php object class.
	 * @param $help_page Name of the help-page
	 * @return string: The created link text
	 */
	function help_link($obj, $help_page = 'help.php')
	{
		$var_str	= "";
		$ret_str	= "";
		$url_str	= $help_page."?page=".basename($_SERVER['PHP_SELF'])."&amp;obj_type=".$obj->type;

		$vars 		= $this->get_page_vars($obj);
		$var_str 	= implode("&amp;page_vars[]=", $vars);
		$url_str	.= "&amp;page_vars[]=".$var_str;
		$ret_str	= "<p><img src=\"images/icons/arrow.gif\" alt=\"\" /> &nbsp;<a title=\"Need help?\" href=\"".$help_page."\" onclick=\"openwindow('".$url_str."', 500, 600, 0, 1, 1);return false;\">Need help?</a>";
		return $ret_str;
	}

	/**
	 * Draws the border and frames of the object listing in the gui.
	 * @param $obj A Nagios-specific php object class.
	 */
	function draw_command_bar($obj)
	{
		/* print the 'command bar' only for non-new objects.
		 * Conditional commandbar items goes first, so ppl recognize
		 * where they are. */
		if (empty($obj->topic)) {
			$obj->topic = $obj->get_object_name();
		}
		echo "<table class='max header' summary=\"Header Table\" style=\"width: 100%\"><tr><td class='CmdBar'>" . $obj->topic . " </td>\n";

		$cmd_order = array
			('dependencies', 'escalations', 'advanced', 'simple','clone', 'copy', 'propagate', 'delete', 'tryout');
		$cmd_links = $obj->get_cmd_bar_links();
		foreach ($cmd_order as $cmd) {
			if (!isset($cmd_links[$cmd]))
				continue;
			echo "<td class='CmdBarItem'>" .
				$cmd_links[$cmd] . $cmd . "</a></td>\n";
		}

		echo "  </tr></table>\n\n";
	}


	/**
	 * Print out the contents of an array as hidden variables
	 * @param $name Base name of the array
	 * @param $ary The variable array in 'key => value' form
	 */
	function hidden_var_array($name, $ary)
	{
		foreach ($ary as $k => $v) {
			if (is_array($v))
			  $this->hidden_var_array($name . '[' . $k . ']', $v);
			else
			  $this->hidden_var($name . '[' . $k . ']', $v);
		}
	}

	/**
	 * Print message to user
	 */
	function print_message($input=false, $left_margin=false)
	{
		if (!$input){
			return false;
		}

		if (is_string($input)){
			// Wrapper for string messages to user
			if ($left_margin)
				echo "<div style=\"margin-left:{$left_margin}ex;\">$input</div>";
			else
				echo $input;
			return true;
		}
		if (is_array($input)){
			// possibly taking hash arrays as input
			foreach ($input as $key => $value) {
				self::print_message($value, $left_margin === false ? 0 : $left_margin + 2);
			}
		}
		return true;
	}

	/**
	 * Draw the object selector that is on top of most object editing pages
	 */
	function draw_object_selector($obj)
	{
		$only_master_object_select = false;

		if ($obj->master_obj_type) {
			$this->form_start('foo_form','get');
			$this->hidden_var('obj_type', $obj->type);
			if (!$obj->master_obj_id)
				$only_master_object_select = true;

			require_once('widgets/select.php');
			// 3 is how badly I want visible filter thingy
			echo '<table id="parent_selector"><tr><td rowspan="2" valign="bottom">';
			select_widget::addFilterlistButtons(3);
			echo '</td>';
			if (count($obj->master_obj_candidates) > 1)
				foreach ($obj->master_obj_candidates as $obj_type => $_)
					echo "<td class=\"header\">$obj_type</td>";
			else
					echo "<td></td>";
			echo '<td rowspan="2" valign="bottom">';
			$this->submit_button('Go', 'action', true);
			echo '</td></tr><tr>';
			foreach ($obj->master_obj_candidates as $obj_type => $_) {
				$select_list = $obj->get_objects_for_user($obj_type);
				if (is_array($select_list)) {
					foreach ($select_list as $k => $v) {
						if (!$obj->can_user_write($obj_type, $k))
							unset($select_list[$k]);
						else
							$select_list[$k] = get_object_name_by_id($obj_type, $k);
					}
				}
				$select_list[0] = ''; // safe, 'cause get_objects_for_user returns a id=>name map, and 0 is not a valid id
				echo '<td>';
				select_widget::make_selection($select_list, $obj_type === $obj->master_obj_type ? $obj->master_obj_id : 0, "master_obj_id[$obj_type]", 'selector_parent');
				select_widget::addFilterlist("master_obj_id[$obj_type]", sizeof($select_list));
				echo '</td>';
			}
			echo '</tr></table>';

			$this->form_end();
			if ($only_master_object_select) {
				$this->print_related($obj);
				$this->html_end();
			}

			?>
			<script type="text/javascript">
			$('.selector_parent').change(function() {
				$(this).parents('table').find('.selector_parent').not(this).each(function() {this.value = 0});
			});
			</script>
			<?php
			echo '<br />';
		}
		$select_obj_type = $obj->type;
		$select_name = 'obj_id';
		$select_default = $obj->id;
		// A master object exists, get slaves for that master
		if (!empty($obj->master_obj_id))
			$select_list = $obj->get_fellow_slaves();
		// No master objects exists, get objects the user can see
		else
			$select_list = $obj->get_objects_for_user($select_obj_type);
		if (is_array($select_list)) {
			foreach ($select_list as $k => $v) {
				//Propagate the list if the user can view the object.
				if (!$obj->can_user_read($select_obj_type, $k))
					unset($select_list[$k]);
				else if ($obj->type !== 'service')
					$select_list[$k] = get_object_name_by_id($select_obj_type, $k);
			}
		}
		//Can user create a new object of type. NACOMA_ACCESS_ADD_DELETE
		if ($obj->can_user_write($select_obj_type, 'new'))
			$select_list['new'] = "Add new $obj->type";

		$this->form_start('bar_form', 'get');
		$this->hidden_var('obj_type', $obj->type);
		if (isset($obj->master_obj_id))
			$this->hidden_var('master_obj_id', $obj->master_obj_id);

		$this->make_filtered_selection($select_list, $select_default, $select_name);

		$this->submit_button('Go', 'action', true);

		if ($obj->master_obj_type)
			$this->hidden_var('master_obj_type', $obj->master_obj_type);
		$this->form_end();
		if ($only_master_object_select) {
			$this->print_related($obj);
			$this->html_end();
		}
	}

	/**
	 * Render a multi-select widget
	 */
	public function make_multi_selection($tokenvalues, $dflt, $select_name = 'selection')
	{
		require_once('widgets/mselect.php');
		mselect_widget::make_multi_selection($tokenvalues, $dflt, $select_name, $this->form_name);
	}

	/**
	 * Render a select widget
	 */
	public function make_selection($tokenvalues, $dflt, $select_name = 'selection', $class_name = 'xlarge')
	{
		require_once('widgets/select.php');
		select_widget::make_selection($tokenvalues, $dflt, $select_name, $class_name);
	}
}
