<?php
require_once('object_class.php');

/**
 * A service runs on a host. The term "service" is used very loosely. It can
 * mean an actual service that runs on the host (POP, SMTP, HTTP, etc.) or
 * some other type of metric associated with the host (response to a ping,
 * number of logged in users, free disk space, etc.).
 */
 class service_class extends object_class
{
	/** type of the object */
	var $type = 'service';
	/** The type of object services must live 'under' */
	var $master_obj_type = 'host';
	var $master_obj_candidates = array('host' => 'host_name', 'hostgroup' => 'hostgroup_name');
	/** variable key identifying our 'master' object */
	var $master_obj_identifier = 'host_name';
	/** The numerical value of the host which is our 'master' */
	var $master_obj_id = false;
	/** What Nagios considers default options */
	var $nagios_defaults =
		array('stalking_options' => array('n' => ''),
		      'notification_options' => array('n' => ''),
		      'flap_detection_options' => array('o' => 1, 'c' => 1, 'w' => 1, 'u' => 1),
		      'parallelize_check' => '1',
		      'obsess' => '1',
		      'check_freshness' => '0',
		      );
	var $slave_types = array(
		'serviceextinfo',
		'serviceescalation',
		'servicedependency');

	/**
	 * Fetches a list of all objects sharing the same master object
	 * @returns An array of service_id => service_description entries
	 */
	function get_fellow_slaves()
	{
		if (!$this->master_obj_id)
			return false;

		$ret = array();
		$query = "SELECT id, service_description FROM service WHERE $this->master_obj_identifier = $this->master_obj_id AND id ".get_exclude_pending_string('service')." ORDER BY service_description";
		$result = pdo_exec_query($query);
		foreach ($result->fetchAll(PDO::FETCH_NUM) as $row) {
			if ($this->can_user_read('service', $row[0]))
				$ret[$row[0]] = $row[1];
		}
		unset($result);
		if ($this->master_obj_type == 'host') {
			// Magick. See explanation in object_class
			$query = "SELECT $this->db_table.id, service_description FROM $this->db_table INNER JOIN hostgroup ON $this->db_table.hostgroup_name = hostgroup.id INNER JOIN host_hostgroup ON hostgroup.id = host_hostgroup.hostgroup WHERE host_hostgroup.host = $this->master_obj_id AND $this->db_table.id ".get_exclude_pending_string($this->type)." ORDER BY service_description";
			$result = pdo_exec_query($query);
			while($row = $result->fetch(PDO::FETCH_NUM)) {
				$ret[$row[0]] = $row[1];
			}
			unset($result);
		}
		$rights_subset = get_objects_for_user('service');
		foreach ($ret as $k => $v) {
			if (!isset($rights_subset[$k]))
				unset($ret[$k]);
		}
		return $ret;
	}

	/**
	 * Validates an object by calling the parent function, as well
	 * as making sure no host gets two services with the same
	 * service_description assigned to it.
	 */
	function validate_object()
	{
		$idx = get_object_index_table($this->master_obj_type);

		$result = parent::validate_object();

		if (!empty($this->obj['service_description'])) {
			# The weird escapings means åäöüÅÄÖÜ.
			# We probably want to generalize this.
			if (!preg_match("/^[ a-zA-Z0-9\x{e5}\x{e4}\x{f6}\{fc}\x{c5}\x{c4}\x{d6}\x{dc}#\/\\:@_.\[\]{}-]+$/u", $this->obj['service_description'])) {
				$this->validation_error('service_description', "Illegal characters in service_description");
			}
		}

		# templates don't need the additional checks below
		if ($this->is_template)
			return $result;

		# these are marked as 'required', so the validation error list
		# will already have them listed.
		if (empty($this->obj[$this->master_obj_identifier]) || empty($this->obj['service_description']))
			return $result;

		$query = "SELECT id FROM service WHERE $this->master_obj_identifier = " .
			pdo_quote_string($this->master_obj_id) . " AND service_description = " .
			pdo_quote_string($this->obj['service_description']);
		if ($this->id != 'new')
			$query .= " AND id != '$this->id'";

		$res = pdo_exec_query($query);
		if ($res->fetch(PDO::FETCH_NUM)) {
			$this->validation_error
				('service_description',
				 "The $this->master_obj_type '" . $idx[$this->master_obj_id] . "'" .
				 " already has a service named '" . $this->obj['service_description'] .
				 "' associated with it");
		}
		unset($res);
		return empty($this->validation_error_list) && $result;
	}

	/**
	 * Loads a service-object based on its Nagios-name
	 *
	 * @return The loaded object, or false on failures
	 */
	function get_object_by_name($name)
	{
		if (!strstr($name, ';'))
			return parent::get_object_by_name($name);

		$tmp_parts = explode(';', $name);
		$master_obj_id = $tmp_parts[0];
		if (!(int)$master_obj_id || is_string($master_obj_id)) {
			# master_obj_id is a string and should be looked up
			$class_name = $this->master_obj_type ."_class";
			$host_obj = new $class_name();
			$host_obj = $host_obj->get_object_by_name($master_obj_id);
			$master_obj_id = $host_obj['id'];
		}
		$obj_name = $tmp_parts[1];
		$query = 'SELECT id FROM ' . $this->type .
			' WHERE service_description= ' .
			pdo_quote_string($obj_name) . ' AND '.$this->master_obj_identifier.' =' . pdo_quote_string($master_obj_id);
		$result = pdo_exec_query($query);
		$row = $result->fetch(PDO::FETCH_NUM);
		unset($result);
		if (!$row) {
			if ($this->master_obj_type === 'host') {
				// perhaps it's a confused user, and this is a hostgroup service
				$query = "SELECT $this->db_table.id FROM $this->db_table INNER JOIN hostgroup ON $this->db_table.hostgroup_name = hostgroup.id INNER JOIN host_hostgroup ON hostgroup.id = host_hostgroup.hostgroup WHERE host_hostgroup.host = ".pdo_quote_string($master_obj_id)." AND service_description=".pdo_quote_string($obj_name)." LIMIT 1";
				$result = pdo_exec_query($query);
				$row = $result->fetch(PDO::FETCH_NUM);
				unset($result);
				if (!$row)
					return false;
				$this->set_master_object_type('hostgroup');
			}
			return false;
		}
		$this->id = $row[0];
		return $this->get_object();
	}

	function denormalize($obj = false)
	{
		$obj = parent::denormalize($obj);

		if (empty($obj[$this->master_obj_identifier]))
			return $obj;

		$idx = get_object_index_table($this->master_obj_type);
		if (isset($idx[$obj[$this->master_obj_identifier]]))
			$obj[$this->master_obj_identifier] = $idx[$obj[$this->master_obj_identifier]];

		return $obj;
	}

	function save_object($user=true)
	{
		$ret = parent::save_object($user);
		if (!$ret)
			return $ret;
		$sub_objs = false;

		$dep_objs = $this->find_slave_objects('serviceextinfo');
		if (isset($dep_objs)) {
			foreach ($dep_objs as $extinfo_id) {
				$extinfo = new serviceextinfo_class($extinfo_id);
				$extinfo->banish_object();
			}
		}

		return $ret;
	}

	public function get_cmd_bar_links()
	{
		$cmd_links = parent::get_cmd_bar_links();
		// instead of duplicating validation from parent, just check if the validation ended up allowing copying, and only then add clone
		if (isset($cmd_links['copy']))
			$cmd_links['clone'] = '<a title="Clone services" href="edit_special.php/copy_slaves?obj_type='.$this->type.'&amp;master_obj_id='.$this->master_obj_id.'&amp;master_obj_type='.$this->master_obj_type.'&amp;obj_id='.$this->id.'">';
		return $cmd_links;
	}

	/**
	 * Given a preexisting service id, returns the type of the parent object
	 */
	public static function get_parent_type($id) {
		$query = "SELECT 1 FROM service WHERE id = ".pdo_quote_string($id)." AND hostgroup_name IS NOT NULL AND hostgroup_name != ''";
		$res = pdo_exec_query($query);
		$row = $res->fetch(PDO::FETCH_NUM);
		if ($row)
			$parent_type = 'hostgroup';
		else
			$parent_type = 'host';
		unset($res);
		return $parent_type;
	}
}
